<?php
/**
 * @package     CSVI
 * @subpackage  CSVI
 *
 * @author      RolandD Cyber Produksi <contact@rolandd.com>
 * @copyright   Copyright (C) 2006 - @year@ RolandD Cyber Produksi. All rights reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 * @link        https://rolandd.com
 */

namespace csvi\com_csvi\model\export;

defined('_JEXEC') or die;

/**
 * Export custom tables.
 *
 * @package     CSVI
 * @subpackage  CSVI
 * @since       6.0
 */
class Custom extends \CsviModelExports
{
	/**
	 * Export the data.
	 *
	 * @return  void.
	 *
	 * @since   6.0
	 *
	 * @throws  \CsviException
	 */
	protected function exportBody()
	{
		if (parent::exportBody())
		{
			// Build something fancy to only get the fieldnames the user wants
			$userfields   = [];
			$exportfields = $this->fields->getFields();
			$customTables = $this->template->get('custom_table');
			$countTables = count((array) $customTables);

			if (!isset($customTables->custom_table1->table))
			{
				// There is no from table, we can't continue
				throw new \CsviException(\JText::_('COM_CSVI_CUSTOM_EXPORT_NO_TABLENAME_SET'));
			}

			// Get the primary table
			$primaryTable      = $customTables->custom_table1->table;
			$primaryTableAlias = ($customTables->custom_table1->tablealias === '') ? '#__' . $primaryTable : $customTables->custom_table1->tablealias;

			// Group By and Sort selected fields
			$groupFields   = $this->template->get('groupbyfields', [], 'array');
			$groupBy       = [];
			$groupByTables = [];
			$groupByFields = [];

			if (!empty($groupFields))
			{
				foreach ($groupFields as $groupField)
				{
					$groupByTables[] = $groupField->groupby_table_name;
					$groupByFields[] = $groupField->groupby_field_name;
				}
			}

			$sortFields   = $this->template->get('sortfields', [], 'array');
			$sortBy       = [];
			$sortByTables = [];
			$sortByFields = [];
			$sortByOrders = [];

			if (!empty($sortFields))
			{
				foreach ($sortFields as $sortName)
				{
					$sortByTables[] = $sortName->sortby_table_name;
					$sortByFields[] = $sortName->sortby_field_name;
					$sortByOrders[] = $sortName->sortby_field_name_order;
				}
			}

			foreach ($exportfields as $field)
			{
				switch ($field->field_name)
				{
					// Man made fields, do not export them
					case 'custom':
						break;
					default:
						$tableName    = $field->table_name ?: $primaryTable;
						$tableAlias   = ($field->table_alias === '') ? '#__' . $tableName : $field->table_alias;
						$columnHeader = $field->column_header ?: $field->field_name;

						if ($countTables === 1)
						{
							$tableAlias = $primaryTableAlias;
						}

						$userfields[] = $this->db->quoteName($tableAlias . '.' . $field->field_name, $columnHeader);

						$groupTable = array_search($tableName, $groupByTables, true);
						$groupKey = array_search($field->field_name, $groupByFields, true);

						if ($groupTable !== false && $groupKey !== false && $groupTable === $groupKey)
						{
							$groupBy[] = $this->db->quoteName($tableAlias . '.' . $field->field_name);
						}

						$sortTable = array_search($tableName, $sortByTables, true);
						$sortKey   = array_search($field->field_name, $sortByFields, true);

						if ($sortTable !== false && $sortKey !== false && $sortTable === $sortKey)
						{
							$sortBy[] = $this->db->quoteName($tableAlias . '.' . $field->field_name) . ' ' . $sortByOrders[$sortKey];
						}
						break;
				}
			}

			$userfields = array_unique($userfields);
			$query      = $this->db->getQuery(true);
			$query->select(implode(",\n", $userfields));
			$query->from($this->db->quoteName("#__" . $primaryTable, $primaryTableAlias));

			for ($i = 2, $iMax = $countTables; $i <= $iMax; $i++)
			{
				$keyName = 'custom_table' . $i;

				if (isset($customTables->$keyName->table))
				{
					$table          = '#__' . $customTables->$keyName->table;
					$tableAlias     = $customTables->$keyName->tablealias ?? '';
					$tableField     = $customTables->$keyName->field;
					$joinTable      = '#__' . $customTables->$keyName->jointable;
					$joinTableAlias = $customTables->$keyName->jointablealias ?? '';
					$joinTableField = $customTables->$keyName->joinfield;

					$mainTable          = ($tableAlias)
						? $this->db->quoteName($table, $tableAlias)
						: $this->db->quoteName($table);

					$joinMainTableField = ($tableAlias)
						? $this->db->quoteName($tableAlias . '.' . $tableField)
						: $this->db->quoteName($table . '.' . $tableField);

					$joinTableField     = ($joinTableAlias)
						? $this->db->quoteName($joinTableAlias . '.' . $joinTableField)
						: $this->db->quoteName($joinTable . '.' . $joinTableField);

					$joinType = $customTables->$keyName->jointype;
					$query->join(
						$joinType,
						$mainTable
						. ' ON ' . $joinMainTableField .
						' = ' . $joinTableField
					);
				}
			}

			// Group the fields
			$groupBy = array_unique($groupBy);

			if (!empty($groupBy))
			{
				$query->group($groupBy);
			}

			// Sort set fields
			$sortBy = array_unique($sortBy);

			if (!empty($sortBy))
			{
				$query->order($sortBy);
			}

			// Add export limits
			$limits = $this->getExportLimit();

			try
			{
				// Execute the query
				$stm = $this->db->replacePrefix($query->__toString());

				if ((int) $limits['limit'] > 0 || (int) $limits['offset'] > 0)
				{
					$stm .= ' LIMIT ' . (int) $limits['offset'] . ', ' . (int) $limits['limit'];
				}

				$records = $this->pdo->yieldObjects($stm);
				$this->log->add('Export query ' . $stm, false);

				// Check if there are any records
				$logcount = $this->pdo->fetchAffected($stm);

				if ($logcount > 0)
				{
					foreach ($records as $record)
					{
						$this->log->incrementLinenumber();

						foreach ($exportfields as $field)
						{
							$fieldname    = $field->field_name;
							$columnHeader = $field->column_header;
							$fieldvalue   = '';

							// Set the field value
							if (isset($record->$fieldname))
							{
								$fieldvalue = $record->$fieldname;
							}

							// Column header is the field alias, use that for retrieve field value if set
							// This is useful in case of exporting multiple tables with same field name
							if ($field->column_header)
							{
								$fieldvalue = $record->$columnHeader;
							}

							// If there is a date format set, use that on field
							if ($field->field_date_format)
							{
								$fieldvalue = $this->fields->getDateFormat($fieldname, $fieldvalue, $field->column_header, $this->exportFormat);
							}

							if ($field->number_field)
							{
								$fieldvalue = number_format(
									$fieldvalue,
									$this->template->get('export_price_format_decimal', 2, 'int'),
									$this->template->get('export_price_format_decsep'),
									$this->template->get('export_price_format_thousep')
								);

							}

							// Store the field value
							$this->fields->set($field->csvi_templatefield_id, $fieldvalue);
						}

						// Output the data
						$this->addExportFields();

						// Output the contents
						$this->writeOutput();
					}
				}
				else
				{
					$this->addExportContent(\JText::_('COM_CSVI_NO_DATA_FOUND'));

					// Output the contents
					$this->writeOutput();
				}
			}
			catch (\Exception $exception)
			{
				$this->log->add('Error: ' . $exception->getMessage(), false);
				$this->log->addStats('incorrect', $exception->getMessage());
			}
		}
	}
}
